/*
 *    This file is part of wdaemon.
 *
 *    wdaemon is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    wdaemon is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with wademon; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <fcntl.h>
#include <stdlib.h>
#include <linux/input.h>
#include <linux/uinput.h>
#include "uinput.h"

#ifndef UINPUT_FILE
#error Please define UINPUT_FILE in Makefile
#endif

int uinput_create(struct uinput_info *info)
{
	struct uinput_user_dev dev;
	char file[50], *tmp;
	int retval;

	tmp = getenv("UINPUT_DEVICE");
	if (!tmp)
		tmp = UINPUT_FILE;
	snprintf(file, sizeof(file), "%s", tmp);

	info->fd = open(file, O_RDWR);
	if (info->fd < 0) {
		fprintf(stderr, "Unable to open uinput file %s: %s", file,
			strerror(errno));
		return -1;
	}

	memset(&dev, 0, sizeof(dev));

	if (info->init_info(info, &dev))
		goto err;

	retval = write(info->fd, &dev, sizeof(dev));
	if (retval < 0) {
		perror("Unable to create input device");
		goto err;
	}
	if (retval != sizeof(dev)) {
		fprintf(stderr, "Short write while creating input device. "
			"Different versions?\n");
		goto err;
	}

	if (info->enable_events(info, &dev))
		goto err;

	retval = ioctl(info->fd, UI_DEV_CREATE);
	if (retval == -1) {
		perror("Unable to create uinput device: ");
		goto err;
	}

	retval = 0;
out:
	return retval;
err:
	close(info->fd);
	goto out;
}

int uinput_write_event(struct uinput_info *info, struct input_event *ev)
{
	if (write(info->fd, ev, sizeof(struct input_event)) !=
						sizeof(struct input_event)) {
		perror("Error writing uinput event: ");
		return 1;
	}
	return 0;
}

